// Slightly modified QMacNativeWidget example from the Qt 5.15 doc https://doc.qt.io/qt-5/qmacnativewidget.html
// https://github.com/qt/qtbase/blob/5.15.2/src/widgets/doc/snippets/qmacnativewidget/main.mm
//
// 3 embedding modes are defined, you can switch between them by defining EMBEDMODE to one of those values:
// QMACNATIVEWIDGET: use QMacNativeWidget (will only work with Qt 5, as this class was removed from Qt 6)
// FROMWINID: use QWidget::fromWinId
// MINIMAL: just bind a QWidget to a NSView using addSubview, as in the QMacNativeWidget example, but without using QMacNativeWidget

#define QMACNATIVEWIDGET 0
#define FROMWINID 1
#define MINIMAL 2

#define EMBEDMODE FROMWINID

///////////////////////////////////////

#include <QApplication>
#include <QtWidgets>
#if EMBEDMODE == QMACNATIVEWIDGET
#include <QMacNativeWidget>
#endif
#import <AppKit/AppKit.h>

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    NSWindow *window = [[NSWindow alloc] initWithContentRect:NSMakeRect(200, app.primaryScreen()->size().height() - 200, 400, 400)
                        styleMask:NSTitledWindowMask | NSClosableWindowMask
                                  | NSMiniaturizableWindowMask | NSResizableWindowMask
                        backing:NSBackingStoreBuffered defer:NO];

#if EMBEDMODE == QMACNATIVEWIDGET
    QMacNativeWidget *nativeWidget = new QMacNativeWidget();
#else
    QWidget* nativeWidget=new QWidget();
#endif
    nativeWidget->resize(300,300);
    nativeWidget->move(0, 0);
    nativeWidget->setPalette(QPalette(Qt::red));
    nativeWidget->setAutoFillBackground(true);
    QVBoxLayout *layout = new QVBoxLayout();
    QPushButton *pushButton = new QPushButton("An Embedded Qt Button!", nativeWidget);
    pushButton->setAttribute(Qt::WA_LayoutUsesWidgetRect); // Don't use the layout rect calculated from QMacStyle.
    layout->addWidget(pushButton);
    //adding a combo box
    QComboBox* comboBox=new QComboBox(nativeWidget);
    comboBox->addItem("Combo Item 1");
    comboBox->addItem("Combo Item 2");
    comboBox->addItem("Combo Item 3");
    layout->addWidget(comboBox);
    //adding a tool button with a menu
    QToolButton* toolButton=new QToolButton(nativeWidget);
    toolButton->setText("Long Click to Show Tool Menu");
    QMenu* menu=new QMenu(toolButton);
    menu->addAction(new QAction("Toolbutton Action 1"));
    menu->addAction(new QAction("Toolbutton Action 2"));
    menu->addAction(new QAction("Toolbutton Action 3"));
    toolButton->setMenu(menu);
    layout->addWidget(toolButton);
    nativeWidget->setLayout(layout);

    // Adjust Cocoa layouts
    NSView *nativeWidgetView = reinterpret_cast<NSView *>(nativeWidget->winId());
    NSView *contentView = [window contentView];
    [contentView setAutoresizesSubviews:YES];
    [nativeWidgetView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    [nativeWidgetView setAutoresizesSubviews:YES];
    NSView *pushButtonView = reinterpret_cast<NSView *>(pushButton->winId());
    [pushButtonView setAutoresizingMask:NSViewWidthSizable];

    // Add the nativeWidget to the window.
#if EMBEDMODE == QMACNATIVEWIDGET
    [contentView addSubview:nativeWidgetView positioned:NSWindowAbove relativeTo:nil];
#else
    #if EMBEDMODE == FROMWINID
    QWindow* qwindow=QWindow::fromWinId((WId)contentView);
    nativeWidget->windowHandle()->setParent(qwindow);
    #else
    [contentView addSubview:nativeWidgetView positioned:NSWindowAbove relativeTo:nil];
    #endif
#endif
    nativeWidget->show();
    pushButton->show();

    // Show the window.
    [window makeKeyAndOrderFront:window];
    [pool release];

    return app.exec(); // gives us the same behavior in both
}
